#include <iostream>
#include <fstream>
#include <cstdlib>
#include <ctime>

#include <boost/random.hpp>
#include <boost/timer/timer.hpp>

using namespace std;
using namespace boost;

int main ( int argc, char **argv ) {
   unsigned long blksize = 1024 * 1024,     // размер блока записи
                 blknumb,                   // число блоков помещающихся в файле
                 wrnumb = 100;              // число блоков для случайного позиционирования и записи
   if( 1 == argc ) { 
      cout << "Запуск:  " << argv[ 0 ] << " [файл] [число записей] [размер блока записи]" << endl;
      exit( EXIT_SUCCESS );
   }
   if( argc > 2 && atoi( argv[ 2 ] ) > 0 ) wrnumb = atoi( argv[ 2 ] );
   if( argc > 3 && atoi( argv[ 3 ] ) > 0 ) {
      blksize = atoi( argv[ 3 ] );
      char c = argv[ 3 ][ strlen( argv[ 3 ] ) - 1 ];
      if( 'K' == c || 'k' == c ) blksize *= 1024L;
      if( 'M' == c || 'm' == c ) blksize *= 1024L * 1024L;
   }

   char *block = new char[ blksize ];       // подготовка образа блока
   for( unsigned int i = 0; i < blksize; ++i ) block[ i ] = i & 0xFF;
   ofstream to( argv[ 1 ],                  // подготовка базового файла
                ios_base::in | ios_base::out | ios_base::ate );
   if( !to ) perror( "открытие файла" ), exit( EXIT_FAILURE );
   unsigned long long flength;

   to.seekp( 0, ios_base::end );
   flength = to.tellp();
   blknumb = flength / blksize;
   cout << "длина файла " << flength << ", блоков в файле " << blknumb << endl;
   bool *indic = new bool[ blknumb ];       // индикатор записи блока
   for( unsigned long i = 0; i < blknumb; ++i ) indic[ i ] = false;

   mt19937 rng;                             // выбор генератора случайных чисел
// mt11213b rnd;
   rng.seed( static_cast<unsigned>( std::time( 0 ) ) ); // инициализация генератора числом секунд с 1970 года
   /* mt19937 produces integers in the range [0, 2^32-1].
      However, we want numbers in the range [0, blknumb - 1].
      The distribution uniform_int_distribution performs this transformation. 
      Given the parameters 0 and blknumb - 1, uniform_int_distribution 
      can produce any of the values 0, 1, 2, ... blknumb - 1. */
   random::uniform_int_distribution<> dist( 0, blknumb - 1 );

   cout << "продолжительность выполнения случайной записи:" << endl;
   {  boost::timer::auto_cpu_timer t;
      for( unsigned long i = 0; i < wrnumb; ++i ) {
         unsigned long nblk = dist( rng );  // порядковый номер блока 
         to.seekp( nblk * blksize, ios_base::beg );
         to.write( block, blksize );
         if( !to ) perror( "запись в файл" ), exit( EXIT_FAILURE );
         to.flush();
         indic[ nblk ] = true;
      }
   }
   cout << "в случайные позиции файла записано " << wrnumb << " блоков длиной " << blksize << " байт каждый" << endl;

   unsigned long nmod = 0;
   for( unsigned long i = 0; i < blknumb; ++i ) nmod += indic[ i ] ? 1 : 0;
   cout << "из " << blknumb << " блоков файла модифицировано " << nmod << " блоков" << endl;

   to.seekp( 0, ios_base::end );
   flength = to.tellp();
   cout << "длина файла после записи: " << flength << endl;
   to.close();
   delete [] block;
   exit( EXIT_SUCCESS );
} 

